#  TMC_seg000_Start_Demo.R 
#  (c) wwosniok@math.uni-bremen.de

#  Truncated minimum chi-square estimation

#  For method description see:
#  W. Wosniok, R. Haeckel: A new indirect estimation of reference intervals:
#  truncated minimum chi-square (TMC) approach. 
#  https://doi.org/10.1515/cclm-2018-1341

#  For the programme manual: see the text in the 'text' directory
#  Template starting programme for the analysis of single data files.

#  CHANGE HISTORY
#  08.11.2022 Start
# ===========================================================================
# remove old variables (for security)
rm(list=ls(all=TRUE))					 


#Arguments are read
#
args1 <- commandArgs()
ar <- which(args1 == "--args")
#
# To get always the last --args one,
#
ar1 <- ar[length(ar)]
args <- args1[(ar1+1):length(args1)]
# 
INPUT_PAR_FILE <- args[1]
#############################
start.time <- date()
#############################									 

if (INPUT_PAR_FILE != "" && file.exists(INPUT_PAR_FILE)) {
    PARS<-read.csv(INPUT_PAR_FILE,header=FALSE,sep=";",dec=".",fileEncoding="UTF-8")

##############################################################################
    dataname         <- as.character(PARS[ 1,1])                   # name of csv-datafile
    main_folder      <- as.character(PARS[ 2,1])                   # main folder of this tool (slash)
    file_prog        <- as.character(PARS[ 3,1])                   # Pfad des TMC_prog für setwd                
    file_res         <- as.character(PARS[ 4,1])                   # subdirectory results
    file_figs        <- as.character(PARS[ 5,1])                   # subdirectory figures
    file_tabs        <- as.character(PARS[ 6,1])                   # subdirectory tables
    file_temp        <- as.character(PARS[ 7,1])                   # subdirectory temp data
    par_file         <- as.character(PARS[ 8,1])                   # relativer Pfad und Name des DataInfoFile.csv
	figtype          <- as.character(PARS[ 9,1])                   # format of graphics (BMP or JPG or PDF)
    com              <- as.numeric(as.character(PARS[10,1]))       # number of decimal places
    a1               <- as.numeric(as.character(PARS[11,1]))       # min age
    a2               <- as.numeric(as.character(PARS[12,1]))       # max age
    age_type         <- as.numeric(as.character(PARS[13,1]))       # type of age interval or NA 
    testunit         <-as.character(PARS[14,1])                    # unit of testvalue (for praphical output)
    head             <-as.character(PARS[15,1])                    # header TRUE/FALSE
    head             <-ifelse(head=="TRUE",TRUE,FALSE)	
	flag_file        <-as.character(PARS[16,1])                    # absolute path and filename of flag file for RLE 
    error_file       <-as.character(PARS[17,1])                    # absolute path and filename of error file for RLE 
    savename         <-as.character(PARS[18,1])                    # absolute path and file name of result file for RLE
##############################################################################																						 
cat("\n dataname              = ", dataname)
cat("\n main_folder           = ", main_folder)
cat("\n file_prog             = ", file_prog)
cat("\n file_res              = ", file_res)
cat("\n file_figs             = ", file_figs)
cat("\n file_tabs             = ", file_tabs)
cat("\n file_temp             = ", file_temp)
cat("\n par_file              = ", par_file)
cat("\n figtype               = ", figtype)
cat("\n com                   = ", com)
cat("\n a1                    = ", a1)
cat("\n a2                    = ", a2)
cat("\n age_type              = ", age_type)
cat("\n Unit of               = ", testunit)
cat("\n Header                = ", head)
cat("\n Flag/Result file txt  = ", flag_file)
cat("\n Error File  csv       = ", error_file)
cat("\n Result File csv       = ", savename)
cat("\n\n")							   
}																								
##############################################################################
# declare the path for R (needed for finding Pfad für R damit R auch die Programme findet
tmc_prog <- paste(main_folder, file_prog, sep="/")
setwd(tmc_prog)
#setwd("C:/TMC13a/TMC_prog")

##############################################################################
# Read external files

source("TMC_seg010_User.R")            # clean up & define user characteristics
source("TMC_seg015_DefaultSettings.R") # Default parameters

#info.file <- "/data/DataFileInfo.csv"        # default 
info.file <- "../temp/DataFileInfo.csv"      

#FileNo     <-  9999        # Test data, similar to AST
#FileNo     <-  8888        # test_na used by RLE
FileNo      <-  7777        # Standarddatensatz des RLE

scale.fact <-     1        # scaling factor for measurand

x.lo.limit <-    NA        # Exclude values < x.lo.limit from all analysis
x.hi.limit <-    NA        # Exclude values > x.hi.limit

use.oh     <-    NA        #  Select outpatient /  hospitalized
use.dev    <-    NA        #  Select analytical device 

                           #  Select time range within a day 
ana.hour.min <- NA         #  Example: 06
ana.minu.min <- NA         #  Example: 30
ana.hour.max <- NA         #  Example: 18
ana.minu.max <- NA         #  Example: 45 

                           #  Select weekdays:
#use.wday <- "all"          #  "all"   or a subset of 
                           #  c("mo", "tu", "we", "th", "fr", "sa", "su")
                           #  e.g.   use.wday <- c("sa", "su")

                           #  Split data be date-time
split.date <- NA           # Example: "04.02.2019", format is "dd.mm.yyyy"
split.time <- NA           # Example: "13:25",      format is "hh:mm"
use.split  <- NA           # Select par to use
                           # 1: before /  2: after given date-time
                        
                           #  Alternative time selection by dates:
                           #  use only data between start.date, end.date, 
                           #  limits included
                           #       format: "dd.mm.yyyy"
start.date <- NA           #  "01.01.2022" 
end.date   <- NA           #  "30.11.2022"


# toDo - transfer in parfile
#age.limits <- c(18, 30, 40, 50, 60, 70, 80, 90, 100)

    # a1 = min age
	# a2 = max age
	# age_type = Schrittweite der Gruppen
	age.limits <- seq(a1,a2,age_type)
	

#   Alternative age definition. Classes may overlap.
#age.class  <- matrix(c(18, 29,
#                       30, 49,
#                       50, 69,
#                       70, 100), byrow=TRUE, ncol=2)
#colnames(age.class) <- c("lo", "hi")

#  Select subgroups (strata) to analyze
ana.type1 <- TRUE            # full dataset            
ana.type2 <- TRUE            # by sex group, all ages  
ana.type3 <- TRUE            # all sexes, by age group 
ana.type4 <- TRUE            # by sex*age group  

#  Clip range of measurand in plots. Does not restrict analysis. 
#  Must be given in scaled units
#  kann als Variable vom RLE übernommen werden
x.clip.min   <-     NA #  0    # left limit
x.clip.max   <-     NA # 80    # right limit
x.clip.by1   <-     NA # 10    # horizontal tickmarks with labels
x.clip.by2   <-     NA #  2    # horizontal tickmarks without labels

#  Define axis for plots of RL vs age
#  a1    :   min age limit from par
#  a2    :   max age limit from par 
age.clip.min   <-  round(a1, digits = -1) #    10  # left limit
age.clip.max   <-  round(a2, digits = -1) #     100  # right limit
age.clip.by1   <-     10  # horizontal tickmarks with labels
age.clip.by2   <-      5  # horizontal tickmarks without labels
 
#  Define RL axis for plots of RL vs age. Must be given in scaled units.
RL.clip.min   <-   NA #100 #    0    # left limit
RL.clip.max   <-   NA #160 #   50    # right limit
RL.clip.by1   <-      10    # horizontal tickmarks with labels
RL.clip.by2   <-       2    # horizontal tickmarks without labels

# --------------------------------------------------------------------------
####  Default settings are in      TMC_seg015_DefaultSettings.R
####  (Nearly) everything can be changed by giving a new value here.
####  Transfer from TMC_seg015_DefaultSettings.R to start file 
# -------------------------------------------------------------------------
#  Graphics file format? (emf, bmp, ...)
# wird inzwischen über args definiert
#figtype <- "bmp"

# Anzahl Dezimalstellen , wird sonst von args übernommen
# wird inzwischen über args definiert
#com <- 0

# Angabe der Einheit der Messergebnisse, wird sonst von args übergeben
#testunit   <- "Unit"       #   wird sonst von args genommen

# Header TRUE/FALSE wird sonst als args übergeben 
#head <- TRUE

# flag_file used in args
#flag_file        <- "C:/TMC14/TMC_prog/result.txt"

# Logdatei definieren
logdat<-paste(main_folder,"/",file_temp, "/RLE_TMC.log", sep="")

# =================================================================
#  Start execution
# =================================================================
# ----------------------------------------------------------------------------
#  Version identification

VersionNr    <- 14
RevisionDate <- "2023-01-02"
sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
cat("\n*********************************************************************",
    "\n  TMC: Truncated Minimum Chi-Square Estimation of Reference limits",
    "\n             Version ", VersionNr, "  Revision ", RevisionDate,
	"\n                          - START - ",
	"\n                           ",date(), 
    "\n*********************************************************************",
    "\n\n")
sink()

####### erst jetzt starte das Hauptprogramm 

source("TMC_seg050_Master.R")      # Master file, controls execution

# =================================================================
#  flag_file schreiben (noch ohne Error-Handling)
# =================================================================
errmess<-"no error"
write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)

# =================================================================
# =====  Programme end  ===========================================
# ================================================================
