###-  Program1 to estimate distribution of non-pathological values from a patient data    -###
###-  Written on 10.03.2010 by Farhad Arzideh (Ar)                                        -###
###-  other authors: Bernd Wolters (Wo)                                                   -###
###   01.06.2014 (Wo): cat included                                                        ###
###   01.08.2014 (Wo): null hypthesis and CI testing optional, graphics format optional    ###
###   18.12.2014 (Ar): error messages are included                                         ###
###   12.01.2015 (Ar): columns for sex and station are redefined; e.g. all rows with       ###
###                    entries "M" or "M " or "M  " transfered to "M" and thereby imported ###
###   29.01.2015 (Wo): argument main_folder unused => free for use by OSM                  ###
###   02.02.2015 (Ar): modified to apply the tool (as default) in windows-system           ###
###                    or alternatively in linux-system (e.g. OSM)                         ###
###   20.02.2015 (Ar): estimation of bandwidth (bw) of kernel density function was modified.#
###   17.04.2015 (Ar): error handling of kde-estimation was modified (line 493-510).       ###
###   23.08.2015 (Ar): the age range of the analysed data set is given in outputs.         ###
###   24.08.2015 (Ar): the results of analysis (tables and graphs) are saved (additionally)### 
###                    in a subfolder of "results folder", created by this program, with  a### 
###                    specific name deduced from the name of data, age range of the       ### 
###                    analysed data and date of the analysis.                             ### 
###   25.08.2015 (Ar): denotation of columns for sex and hospital was changed (row 219)    ###
###   25.08.2015 (Ar): error handling was modified to catch and deliver mor details.       ###
###   26.08.2015 (Ar): a subfolder with a special name, generated by the name of the       ###
###                    test value (measurand) + age range + date of analysis, is created   ###
###                    in the   \results folder (if it does not exist). The evaluations    ###
###                    results and graphs) are saved in a subfolder \step1 created         ###
###                    in this subfolder.                                                  ###
###   26.08.2015 (Ar): the estimation of bandwidth for kernel density estimation is        ###
###                    modified for the cases that the sample size is too large.           ### 
###   09.02.2016 (Ar): y-limits of graphical displaying for time dependence of values      ###
###                    for male and female subjects are unified.                           ###
###   22.07.2016 (Ar): "error message: minimum age can not be greater than its maximum!"   ###
###   25.07.2016 (Ar): A report data, prog_log.txt, will be created and saved in           ###
###                      "result"-file.                                                    ###
###   19.09.2016 (Ar): special symobols ($, &, /, etc. and empty spaces) in the dataname   ###
###                    are replaced with underline (_).                                    ###
###   19.09.2016 (Ar): choice of X-axis range was modified: if (x2==0 | x2 <= x1)          ### 
###   12.10.2016 (Wo): age-count graphic integrated (Hr. Geissler, OSM)                    ###
###                    testunit (new) and testname in all graphics                         ###
###   18.10.2016 (Ar): new variables are defined to analyse the data in a date range       ###
###   19.10.2016 (Ar): i) new variables are defined to analyse the data in a               ###
###                       given date range                                                 ### 
###                    ii) a copy of data with used columns named data_step1.csv           ###
###                        will be saved in a subdirectory of result file.                 ###
###   12.02.2017 (Wo): saving of plots changed (incl. plot_wrapper.r)                      ###
###   01.03.2017 (Wo): logfile now for every step                                          ###
###   23.07.2017 (Wo): logfile renamend to RLE_Prog1.log                                   ###
###   04.09.2017 (Ar): program can be run with time limitations (date1=start date,         ###
###                    or/and date2=end date)                                              ###
###   08.09.2017 (Wo): logfile in Subdirectory of temp data                                ###
###   28.09.2017 (Wo): denotation of header harmonized                                     ###
###   02.05.2018 (Wo): more changes of header (SD instead of Standard deviation)           ###
###   07.05.2018 (Wo): file_data renamed file_temp for temp subdirectory                   ###
###                    file_data now used for classic only                                 ###
###   01.05.2023 (Wo): try for all needed libraries included                               ###
###   03.07.2023 (Ar/Wo): read.csv fileEncoding = "UTF-8" introduced, to handle            ###
###                    German Umlaute                                                      ###
############################################################################################## 

rm(list=ls(all=TRUE))

#*************************#
###-  DATA locations   -###
#*************************#

############################################
############################################
###                                      ###
###   1. gives path of the par data      ###
###                                      ###
############################################
############################################
#
cat("read arguments (1.) ....\n")

#Arguments are read
#
args1 <- commandArgs()
ar <- which(args1 == "--args")

#
# To get always the last --args one,
#
ar1 <- ar[length(ar)]
args <- args1[(ar1+1):length(args1)]

#
# List args contains now the real arguments
#
#
# DATA locations    
#
INPUT_PAR_FILE <- args[1]
start.time <- date()
##############################################################
##############################################################
###                                                        ###
###   2. assignment of variables in R from the par-data    ###
###                                                        ###
##############################################################
##############################################################
#
#####################
tryCatch({
#####################
### Look for INPUT-PARAMETERFILE and Set Parameters
if (INPUT_PAR_FILE != "" && file.exists(INPUT_PAR_FILE)) {
    parFileReport<-1   #Fehlerbearbeitung nicht realisiert
    PARS<-read.csv(INPUT_PAR_FILE,header=FALSE,sep=";",dec=".",fileEncoding="UTF-8")

############################################

    cat("assingment of variabels (2.) ....\n")    

#####################################################################################
    dataname<-as.character(PARS[1,1])                # name of csv-datafile
    head<-as.character(PARS[2,1])                    # header TRUE/FALSE
    head<-ifelse(head=="TRUE",TRUE,FALSE)
    c2<-as.character(PARS[3,1])                      # column of measurand
    c1<-as.character(PARS[4,1])                      # column of age
    c3<-as.character(PARS[5,1])                      # column of gender
    a1<-as.numeric(as.character(PARS[6,1]))          # min age
    a2<-as.numeric(as.character(PARS[7,1]))          # max age
    m<-as.character(PARS[8,1])                       # male character
    f<-as.character(PARS[9,1])                       # female character
    com<-as.numeric(as.character(PARS[10,1]))        # number of decimal places
    dec.sep<-as.character(PARS[11,1])                # decimal separator
    quant<-as.numeric(as.character(PARS[12,1]))      # quantiles
    model<-as.character(PARS[13,1])                  # model
    #nmc<-as.numeric(as.character(PARS[14,1]))        # not used! number of MC simulations for estimation of KS-distance
    #pathol<-as.character(PARS[15,1])                 # pathological value low/high/both(low+high)
    x1<-as.numeric(as.character(PARS[16,1]))         # min value of x-axis
    x2<-as.numeric(as.character(PARS[17,1]))         # max value of x-axis
    c4<-as.character(PARS[18,1])                     # column of hospitalized/ambulatory
    hospit<-as.character(PARS[19,1])                 # denotation of hospitalized/ambulatory
    #alpha1<-as.numeric(as.character(PARS[20,1]))     # not used! value of alpha error for the test statistic
    testvalue<-as.character(PARS[21,1])              # name of measurand/quantity
    #ci<-as.character(PARS[22,1])                     # not used! confidence intervals TRUE/FALSE
    nrep<-as.numeric(as.character(PARS[23,1]))       # number of bootstraps for CI
    #l0<-as.numeric(as.character(PARS[24,1]))         # min value of transformation parameter
    #n<-as.numeric(as.character(PARS[25,1]))          # start value of transformation parameter
    minsize<-as.numeric(as.character(PARS[26,1]))    # minimum of sample size in each group
    flag_file<-as.character(PARS[27,1])              # result file
    #free_variable<-as.character(PARS[28,1])          # free for other user eg OSM    
    file_prog<-as.character(PARS[29,1])              # subdirectory program
    file_res<-as.character(PARS[30,1])               # subdirectory results
    file_temp<-as.character(PARS[31,1])              # subdirectory temp data
    main_folder1<-as.character(PARS[32,1])           # main folder of this tool (slash)
    #stat_test<-as.character(PARS[33,1])              # statistical test TRUE/FALSE
    graph_format<-as.character(PARS[34,1])           # format of graphics (BMP or JPG or PDF)
                                                     # for using the tool by OSM-system
                                                     # one should set it by "OSM" 
    error_file<-as.character(PARS[35,1])             # absolute path and filename of error file
    savename1<-as.character(PARS[36,1])              # absolute path and file name of result1 file
    #savename2<-as.character(PARS[37,1])              # absolute path and file name of result2 file - only used in program2
    r_func<-as.character(PARS[38,1])                 # absolute path for r functions   
    testunit<-as.character(PARS[39,1])               # unit of testvalue (for praphical output) 
    #  group<-as.character(PARS[40,1])               # defines subgroups to evaluate: n.d.
    c5<-as.character(PARS[41,1])                     # column of date of test
    c6<-as.character(PARS[42,1])                     # format of date of test
    date1<-as.character(PARS[43,1])                  # start date of data to be analysed 
    date2<-as.character(PARS[44,1])                  # end date of data to be analysed
    # PARS[45,1] not used in program1 
    file_data<-as.character(PARS[46,1])              # subdirectory for data (identically with temp if standard dataset used)
    ###########################################
    }
##############################################
par_file<-paste(main_folder1,"/",file_temp,"/", sep="")   # complete path of subdirectory of par with slash
data_file<-paste(main_folder1,"/",file_data,"/", sep="")  # subdirectory of data (for classic only)
result_file<-paste(main_folder1,"/",file_res,"/", sep="") # complete path of subdirectory of results  with slash
prog_file<-paste(main_folder1,"/",file_prog,"/", sep="")  # complete path of subdirectory of prog with slash
##############################################
# create subfolders in folder \results to save the results and graphs. 
testvalue<-gsub('([[:punct:]])|\\s','_',testvalue)        # replace special symobols ($, &, /, etc. and empty spaces) with underline (_). 
result_file0<-paste(result_file, testvalue,"age",a1,a2, "date",format(Sys.time(), "%Y-%m-%d"), sep = "_") 
dir.create(result_file0, showWarnings = TRUE)
result_file1<-paste(result_file0 , "/","step1", sep = "")
dir.create(result_file1, showWarnings = TRUE)
##############################################
num.sink <- sink.number()
if (num.sink > 0)  for (i in 1:num.sink)  sink() 
   logdat<-paste(main_folder1,"/",file_temp, "/RLE_Prog1.log", sep="")
   })
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: the entries in tab parameter for analysis in start dialog box are not defined, completely or correctely. Or the user has no admin permission to create subfolders"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
##############################################################
###############################################################
###                                                         ###
###               3. read of data                           ###
###                                                         ###
###############################################################
###############################################################
{
if (length(PARS[1,])==1) {      # if only one data should be analysed
  file.name<-paste(data_file,dataname, sep="")
  check<-1
  suppressWarnings(data<-try(read.table(file=file.name,dec=dec.sep,header=head,sep=";"), TRUE))
  if (class(data) == "try-error")  {
    errmess<-"error: either data set does not exist, or entries in parameter for analysis are not defined correctely or data in each line are not separated by ; or entries have not the same class."
    write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
    write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
    quit(save = "no", status = 0, runLast = FALSE)
    }else{
    data<-read.table(file=file.name,dec=dec.sep,header=head,sep=";")
    }
  }else{
  # if more than one data should be analysed
  dataname<-PARS[1,]
  J<-length(dataname)
  data<-NULL
  for (j in 1:J)  {
    check<-2
    dataname1<-dataname[[j]]
    file.name<-paste(data_file,dataname1, sep="")
    suppressWarnings(data1<-try(read.table(file=file.name,dec=dec.sep,header=head,sep=";"), TRUE))
    if (class(data1) == "try-error"){
      errmess<-"error: either data set does not exist, or entries in parameter for analysis are not defined correctely or data in each line are not separated by ; or entries have not the same class."
      write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
      write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
      quit(save = "no", status = 0, runLast = FALSE)
      }else{
      data1<-read.table(file=file.name,dec=dec.sep,header=head,sep=";")
      suppressWarnings(data<-try(rbind(data,data1),TRUE))
      if (class(data) == "try-error")  {
        errmess<-"error: the datasets have different structure and they can not be analysed together"
        write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
        write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
        quit(save = "no", status = 0, runLast = FALSE)
        } 
      }
    }
  }
}
####################################
####################################
# rename of columns of data from letters to numeric (e.g. C into 3)
nw<-ncol(data)
v1<-c(LETTERS[1:nw])
v2<-c(1:nw)
mat<-matrix(c(v1,v2),nrow=2,byrow=TRUE)
c1<-as.numeric(mat[,mat[1,]==c1][2])
c2<-as.numeric(mat[,mat[1,]==c2][2])
c3<-as.numeric(mat[,mat[1,]==c3][2])
c4<-as.numeric(mat[,mat[1,]==c4][2])
c5<-as.numeric(mat[,mat[1,]==c5][2])
if (!is.na(c4)) {
  data<-subset(data,subset=(data[,c4]==hospit))
  mycol<-c(c3,c4)
  for (i in mycol) {                                    # columns for sex and station are redefined, that
                                                        # causes e.g. all rows with entries "M" or "M " or "M  " transfered to "M".  
    data[,i] <- gsub(" ", "", data[,i])
    }
  }else{
  data[,c3] <- gsub(" ", "", data[,c3])
  }
###############################################
## load of required libraries:
try(library(msm))
try(library(geoR))
try(library(mgcv))
try(library(survival))
try(library(MASS))
try(library(mvtnorm))
try(library(nlme))
try(library(expm))
try(library(sp))
try(library(splancs))
errmess<-geterrmessage()                                                                       #test whether or not packages exists
if (errmess=="") {
  errmess<-"no error"
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  } else {                                                                       # wenn ein Fehler auftrat
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
cat("all libraries loaded ....\n")                  
###################################################
# test whether the necessary columns are defined:
if (!is.na(c1) & !is.na(c2) & !is.na(c3)) {
  errmess<-"no error"
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  } else {
  errmess<-"error: the columns of value, age or sex are not defined"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
###################################################
###################################################
if (a1 > a2) {            
  errmess<-"error: minimum age can not be greater than its maximum!"
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
###################################################
####################################################
# functions from prog subdirectories are now loaded:
try({
  source(paste(r_func,"/data_sep1.R",sep=""))
  source(paste(r_func,"/data_gr.R",sep=""))
  source(paste(r_func,"/kde.R",sep=""))
  source(paste(r_func,"/kde1.R",sep=""))
  source(paste(r_func,"/graph_hist.R",sep="")) 
  source(paste(r_func,"/plot_wrapper.R",sep=""))                                                      
  })
errmess<-geterrmessage() 
if (errmess!="") {                #   test whether required functions have been loaded.     
  errmess<-"error: one or more required functions to analyse the data could not be loaded. Either subdirectory R functions has not been set correctly or some functions are removed. Create R functions again."
  sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("R version is: ", as.character(getRversion()), "\n\n", sep = "")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
####################################################
sink(file=logdat,append=FALSE, split=TRUE,type = c("output","message"))
cat("================================================================\n\n")
cat("R version is: ", as.character(getRversion()), "\n\n", sep = "")
cat("================================================================\n\n")
cat("Data have been imported correctly","\n\n")
cat("================================================================\n\n")
cat("all functions loaded ....\n\n")
sink()
####################################################
####################################################
#########################
# Time limitation of data:
#########################
tryCatch({
  if(!is.na(date1)) date1<-as.Date(date1,,format=c6)
  if(!is.na(date2)) date2<-as.Date(date2,,format=c6)
  }) 
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: choosed formats for start or end date are false. You can set start and end date by NA"
   sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
   end.time <- date()
   cat("Program start:", start.time,"\n")
   cat("program end:  ", end.time,"\n")
   cat("================================================================\n\n")
   cat(errmess, "\n")
   sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
###############################################
if (!is.na(date1) & !is.na(date2) & date1 >= date2 ) {            
errmess<-"error: start date can not be greater as end date! You can set start and end date by NA"
sink(file=logdat,append=T, split=TRUE,type = c("output","message"))
end.time <- date()
cat("Program start:", start.time,"\n")
cat("program end:  ", end.time,"\n")
cat("================================================================\n\n")
cat(errmess, "\n")
sink()
   write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
   write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
   quit(save = "no", status = 0, runLast = FALSE)
   }
################
case<-0
tryCatch({
  if (!is.na(c5) & !is.na(c6) &  !is.na(date1)) {
    data[,c5]<-as.Date(data[,c5], format=c6)# appended format for date of analysis 
    date1<-as.Date(date1,format=c6)
    data1<-subset(data,subset=(data[,c5] >= date1))
    len.1<-nrow(data1)
    if (len.1!=0 & !is.na(len.1)){
      case<-1
      sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
      cat("================================================================\n\n")
      cat("Time of analysis has been limited to the given start date.","\n\n")
      cat("================================================================\n\n")
      sink()
      data<-data1
      }else{
      errmess<-"error: either choosed format for date or column of date or given start date are not given correctly. Program was run without consider start date."
      sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
      cat("================================================================\n\n")
      cat(errmess, "\n")
      sink()
      }
    }else{
    sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
    cat("================================================================\n\n")
    cat("either start date is not given or no information about date of analysis exists or choosed format/column for date or start date are not given correctly\n")
    cat("Program is running without considering start date.","\n\n")
    cat("================================================================\n\n")
    sink()          
    }
 }, silent = TRUE) 
 #########################
case<-0
tryCatch({
  if (!is.na(c5) & !is.na(c6) &  !is.na(date2)) {
    data[,c5]<-as.Date(data[,c5], format=c6)# appended format for date of analysis 
    date2<-as.Date(date2,format=c6)
    data1<-subset(data,subset=(data[,c5] <= date2))
    len.1<-nrow(data1)
    if (len.1!=0 & !is.na(len.1)){
      case<-1
      sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
      cat("================================================================\n\n")
      cat("Time of analysis has been limited to the given end date.","\n\n")
      cat("================================================================\n\n")
      sink()
      data<-data1
      }else{
      errmess<-"error: either choosed format for date or column of date or given end date are not given correctly. Program was run without consider end date."
      sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
      cat("================================================================\n\n")
      cat(errmess, "\n")
      sink()
      }
    }else{
    sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
    cat("================================================================\n\n")
    cat("either end date is not given or no information about date of analysis exists or choosed format/column for date or end date are not given correctly\n")
    cat("Program is running without considering end date","\n\n")
    cat("================================================================\n\n")
    sink()          
    }
 }, silent = TRUE)   
################################################################ 
###                                                          ###
###   3.b save copy of used data                             ### 
###                                                          ###
################################################################
result.step1<-paste(result_file0, "/","data_step1.csv", sep = "")
if (case==0){          
  colname<-c("Age","Value","Gender")
  write.table(data[,c(c1,c2,c3)], file=result.step1, quote=FALSE, row.names=FALSE,sep=";",col.names=colname)   
  cat("3.b dataset copied in subdirectory ....\n")  
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  cat("================================================================\n\n")
  cat(paste("3.b A copy of data with used columns was put in\n ", result.step1, sep=" "),"\n\n")
  cat("================================================================\n\n")
  sink()
  }else{
  colname<-c("Age","Value","Gender","Date_of_Analysis")
  write.table(data[,c(c1,c2,c3,c5)], file=result.step1, quote=FALSE, row.names=FALSE,sep=";",col.names=colname)   
  cat("3.b dataset copied in subdirectory ....\n")  
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  cat("================================================================\n\n")
  cat(paste("3.b A copy of data with used columns in given date range was put in \n", result.step1, sep=" "),"\n\n")
  cat("================================================================\n\n")
  sink()
  }
####################################################
####################################################
###                                              ###
###   4. Main program                            ###
###                                              ###
####################################################
####################################################
cat("program is running ...............\n")
###################################################
if (is.na(testvalue)) {
  testvalue<-""
  }
####################################################
tryCatch(d<-data.sep(data=data,c1=c1,c2=c2,c3=c3,a1=a1,a2=a2,m=m,w=f))
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: either columns of values/ages/sex do not exist/defined correctely or denotation of sex is not correctly  or dataset does not contain male and female results or limits of ages are not given correctly."
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
##################################################
dbm<-d$all  # values for male+female subjects (age group: a1<= age <= a2)
dmbm<-d$m   # values for male subjects (age group: a1<= age <= a2)
dwbm<-d$w   # values for female subjects (age group: a1<= age <= a2)
if (ncol(dbm)<3 | ncol(dmbm)<3 | ncol(dwbm)<3 | nrow(dbm)==0 | nrow(dmbm)==0 | nrow(dwbm)==0){
  errmess<-"error: either columns of values/ages/sex do not exist/defined correctely or denotation of sex is not correctly  or dataset does not contain male and female results or limits of ages are not given correctly."
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
#####################################
#quantiles:
tryCatch({
  vec<-dmbm[,c1]
  br<-seq(min(vec),max(vec)+1,1)
  mhist<-hist(vec,br=br,right=F,plot=F)
  zbr<-length(mhist$breaks)
  zbr1<-mhist$breaks[-zbr]
  xagem<-zbr1[mhist$counts!=0]
  ##########################
  vec<-dwbm[,c1]
  br<-seq(min(vec),max(vec)+1,1)
  mhist<-hist(vec,br=br,right=F,plot=F)
  zbr<-length(mhist$breaks)
  zbr1<-mhist$breaks[-zbr]
  xagew<-zbr1[mhist$counts!=0]
  ###########################
  vec<-dbm[,c1]
  br<-seq(min(vec),max(vec)+1,1)
  mhist<-hist(vec,br=br,right=F,plot=F)
  zbr<-length(mhist$breaks)
  zbr1<-mhist$breaks[-zbr]
  xage<-zbr1[mhist$counts!=0]
########################################################################
# Age-count graph with male and female (without all)
########################################################################
countagem<-tapply(dmbm[,c2],dmbm[,c1],length)
countagew<-tapply(dwbm[,c2],dwbm[,c1],length)
bildc<-paste(result_file,"age_count", sep="")
bildc.1<-paste(result_file1,"/","age_count", sep="")
open_plot(graph_format=graph_format,file=bildc,width=7,height=5)
Y.lim<-c(min(countagem,countagew), max(countagem,countagew))
plot(xagem,countagem,type="o", col="blue",xlim=c(a1,a2),ylim=Y.lim,xlab="Age",ylab=paste("number of values",paste(testvalue," ",testunit),sep=" - "),main="Counts (male=blue, female=green)")
lines(xagew,countagew,type="o",col="green3",lty=2)
close_plot(graph_format=graph_format,file=bildc)
close_plot(graph_format=graph_format,file=bildc.1)
################################
if (quant ==1)   {
   # for quant=1 only medians of data values for different ages are shown.
   cat("calculating and plotting 50-percentile is running .......\n")
   q50<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.50)
   q50m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.50)
   q50w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.50)
   ylim1<-c(min(q50,q50m,q50w),max(q50,q50m,q50w))               # y-limit defined.
   bild4<-paste(result_file,"age_valuem.",graph_format,sep="")
   bild4.1<-paste(result_file1,"/","age_valuem.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild4,width=7,height=5)
   plot(xagem,q50m,type="o",col="blue",xlim=c(a1,a2),xlab="Age",ylab=paste(testvalue," ",testunit),main="50 percentiles (male)",ylim=ylim1)  # unique y-limit.
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild4)
   close_plot(graph_format=graph_format,file=bild4.1)
   bild5<-paste(result_file,"age_valuew.",graph_format,sep="")
   bild5.1<-paste(result_file1,"/","age_valuew.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild5,width=7,height=5)
   plot(xagew,q50w,type="o",col="blue",xlim=c(a1,a2),xlab="Age",ylab=paste(testvalue," ",testunit),main="50 percentiles (female)",ylim=ylim1)  # unique y-limit.
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild5)
   close_plot(graph_format=graph_format,file=bild5.1)
   bild6<-paste(result_file,"age_value.",graph_format,sep="")
   bild6.1<-paste(result_file1,"/","age_value.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild6,width=7,height=5)
   plot(xage,q50,type="o",col="blue",xlim=c(a1,a2),xlab="Age",ylab=paste(testvalue," ",testunit),main="50 percentiles (all)",ylim=ylim1)  # unique y-limit.
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild6)
   close_plot(graph_format=graph_format,file=bild6.1)
   } else if (quant==3) {
   # for quant=3 the 25-, 50- and 75-percentiles of data values for different ages are shown.
   cat("calculating and plotting 25-,50-, and 75-percentiles is running .......\n")
   q25<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.25)
   q50<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.50)
   q75<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.75)
   q25m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.25)
   q50m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.50)
   q75m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.75)
   q25w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.25)
   q50w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.50)
   q75w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.75)
   ylim1<-c(min(q25,q25m,q25w),max(q75,q75m,q75w))              
   bild4<-paste(result_file,"age_valuem.",graph_format,sep="")
   bild4.1<-paste(result_file1,"/","age_valuem.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild4,width=7,height=5)
   plot(xagem,q75m,type="o",col="red",xlim=c(a1,a2),ylim=ylim1,xlab="Age",ylab=paste(testvalue," ",testunit), main="25(green), 50(blue) and 75(red) percentiles (male)",lty=2)    # unique y-limit.
   lines(xagem,q50m,type="o",col="blue",lty=3)
   lines(xagem,q25m,type="o",col="green3",lty=2)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild4)
   close_plot(graph_format=graph_format,file=bild4.1)
   bild5<-paste(result_file,"age_valuew.",graph_format,sep="")
   bild5.1<-paste(result_file1,"/","age_valuew.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild5,width=7,height=5)
   plot(xagew,q75w,type="o",col="red",xlim=c(a1,a2),ylim=ylim1,xlab="Age",ylab=paste(testvalue," ",testunit),
   main="25(green), 50(blue) and 75(red) percentiles (female)",lty=2)  # unique y-limit.
   lines(xagew,q50w,typ="o",col="blue",lty=3)
   lines(xagew,q25w,typ="o",col="green3",lty=2)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild5)
   close_plot(graph_format=graph_format,file=bild5.1)
   bild6<-paste(result_file,"age_value.",graph_format,sep="")
   bild6.1<-paste(result_file1,"/","age_value.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild6,width=7,height=5)
   plot(xage,q75,type="o",col="red",xlim=c(a1,a2),ylim=ylim1,xlab="Age",ylab=paste(testvalue," ",testunit),
   main="25(green), 50(blue) and 75(red) percentiles (all)",lty=2) # unique y-limit.
   lines(xage,q50,typ="o",col="blue",lty=3)
   lines(xage,q25,typ="o",col="green3",lty=2)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild6)
   close_plot(graph_format=graph_format,file=bild6.1)
   } else {
   # for quant=5 the 5- 25-, 50- and 75- and 95-percentiles of data values for different ages are shown.
   cat("calculating and plotting 5-, 25-, 50-, 75- and 95-percentiles is running ..........\n")
   q05<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.05)
   q25<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.25)
   q50<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.50)
   q75<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.75)
   q95<-tapply(dbm[,c2], dbm[,c1],quantile,prob=0.95)
   q05m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.05)
   q25m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.25)
   q50m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.50)
   q75m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.75)
   q95m<-tapply(dmbm[,c2], dmbm[,c1],quantile,prob=0.95)
   q05w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.05)
   q25w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.25)
   q50w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.50)
   q75w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.75)
   q95w<-tapply(dwbm[,c2], dwbm[,c1],quantile,prob=0.95)
   ylim1<-c(min(q05,q05m,q05w),max(q95,q95m,q95w))               
   bild4<-paste(result_file,"age_valuem.",graph_format,sep="")
   bild4.1<-paste(result_file1,"/","age_valuem.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild4,width=7,height=5)
   plot(xagem,q95m,type="o",col="orange",xlim=c(a1,a2),ylim=ylim1,xlab="Age",ylab=paste(testvalue," ",testunit),
   main="5(black), 25(green), 50(blue), 75(red) and 95(orange) percentiles (male)",lty=3) # unique y-limit.
   lines(xagem,q75m,typ="o",col="red",lty=2)
   lines(xagem,q50m,typ="o",col="blue",lty=3)
   lines(xagem,q25m,typ="o",col="green3",lty=2)
   lines(xagem,q05m,typ="o",col="black",lty=3)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild4)
   close_plot(graph_format=graph_format,file=bild4.1)
   bild5<-paste(result_file,"age_valuew.",graph_format,sep="")
   bild5.1<-paste(result_file1,"/","age_valuew.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild5,width=7,height=5)
   plot(xagew,q95w,type="o",col="orange",xlim=c(a1,a2),ylim=ylim1,xlab="Age",ylab=paste(testvalue," ",testunit),
   main="5(black), 25(green), 50(blue), 75(red) and 95(orange) percentiles (female)",lty=3) # unique y-limit.
   lines(xagew,q75w,typ="o",col="red",lty=2)
   lines(xagew,q50w,typ="o",col="blue",lty=3)
   lines(xagew,q25w,typ="o",col="green3",lty=2)
   lines(xagew,q05w,typ="o",col="black",lty=3)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild5)
   close_plot(graph_format=graph_format,file=bild5.1)
   bild6<-paste(result_file,"age_value.",graph_format,sep="")
   bild6.1<-paste(result_file1,"/","age_value.",graph_format,sep="")
   open_plot(graph_format=graph_format,file=bild6,width=7,height=5)
   plot(xage,q95,type="o",col="orange",xlim=c(a1,a2),ylim=ylim1,xlab="Age",ylab=paste(testvalue," ",testunit),
   main="5(black), 25(green), 50(blue), 75(red) and 95(orange) percentiles (all)",lty=3,cex.main=1) # unique y-limit.
   lines(xage,q75,typ="o",col="red",lty=2)
   lines(xage,q50,typ="o",col="blue",lty=3)
   lines(xage,q25,typ="o",col="green3",lty=2)
   lines(xage,q05,typ="o",col="black",lty=3)
   abline(0,0)
   close_plot(graph_format=graph_format,file=bild6)
   close_plot(graph_format=graph_format,file=bild6.1)
   }
})
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: either entries in tab parameter for analysis are not defined correctely or data in each line are not separated by ; or entries in some columns have not the same class."
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
#########################
# Estimation of Kern density function
###########################
tryCatch({
  dbm<-dbm[,c2]                   # values for male+female subjects (age group: a1< age <= a2)
  dmbm<-dmbm[,c2]                 # values for male subjects (age group: a1< age <= a2)
  dwbm<-dwbm[,c2]                 # values for female subjects (age group: a1< age <= a2)
  n1<-length(dbm)
  n2<-length(dmbm)
  n3<-length(dwbm)
  min1<-round(min(dbm),com)
  min2<-round(min(dmbm),com)
  min3<-round(min(dwbm),com)
  max1<-round(max(dbm),com)
  max2<-round(max(dmbm),com)
  max3<-round(max(dwbm),com)
  mean1<-round(mean(dbm),com)
  mean2<-round(mean(dmbm),com)
  mean3<-round(mean(dwbm),com)
  sd1<-round(sd(dbm),com)
  sd2<-round(sd(dmbm),com)
  sd3<-round(sd(dwbm),com)
  med1<-round(median(dbm),com)
  med2<-round(median(dmbm),com)
  med3<-round(median(dwbm),com)
  })
errmess<-geterrmessage() 
if (errmess!="") {            
  errmess<-"error: mean or median or sd of data could not be calculated."
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
#########################
#########################
tryCatch({
  d<-data.gr(data=data,c1=c1,c2=c2,c3=c3,a1=a1,a2=a2,m=m,w=f)
  dbm<-d$all  # values for male+female subjects (age group: a1<= age <= a2)
  dmbm<-d$m   # values for male subjects (age group: a1<= age <= a2)
  dwbm<-d$w   # values for female subjects (age group: a1<= age <= a2)
  })
errmess<-geterrmessage() 
if (errmess!="" | length(dmbm) < 10 | length(dwbm) < 10) {            
  errmess<-"error: the sample size is small. Try with more data points or wider age range."
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
########################
if (length(dmbm) > 40000){
  bw1<-bw.nrd(dmbm)
  }else{                                
  bw1<-try(bw.SJ(dmbm,method="dpi"),silent=T)
  if (class(bw1) == "try-error"){ 
    bw1<-bw.nrd(dmbm)
    }
  }
if (length(dwbm) > 40000){
  bw2<-bw.nrd(dwbm)
  }else{ 
  bw2<-try(bw.SJ(dwbm,method="dpi"),silent=T)
  if (class(bw2) == "try-error"){
    bw2<-bw.nrd(dwbm)
    }
  }
if (length(dbm) > 40000){
  bw0<-bw.nrd(dbm)
  }else{       
  bw0<-try(bw.SJ(dbm,method="dpi"),silent=T)
  if (class(bw0) == "try-error"){            
    bw0<-min(bw.nrd(dbm),bw1,bw2)
    }
  }       
bw0<-round(bw0,com)
bw1<-round(bw1,com)
bw2<-round(bw2,com)
step0<-(10^(-com))
bwa<-max(bw0,(step0)/2)                    # bandwidth of kde for all data
bwm<-max(bw1,(step0)/2)                    # bandwidth of kde for male
bww<-max(bw2,(step0)/2)                    # bandwidth of kde for female
if (bwa <=0 | bwm <=0 | bww <=0 | is.na(bwa) | is.na(bwm) | is.na(bww)){
  errmess<-"error: bandwidths could not be estimated. Try with more data points."
  sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
  end.time <- date()
  cat("Program start:", start.time,"\n")
  cat("program end:  ", end.time,"\n")
  cat("================================================================\n\n")
  cat(errmess, "\n")
  sink()
  write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
  write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
  quit(save = "no", status = 0, runLast = FALSE)
  }
#########################
stepa<-max(bw0,step0)
###########################
stepm<-max(bw1,step0)
###############
stepw<-max(bw2,step0)
###############################################
if (x1==0) {
   x1<-max(round(mean1-4*sd1,com),0)
   }
if (x2==0 | x2 <= x1) {
   x2<-round(mean1+4*sd1,com)
   }
T<-10*(mean1+2*sd1)
################################################
tryCatch({
  bild1<-paste(result_file,"kde_male.",graph_format,sep="")
  bild1.1<-paste(result_file1,"/","kde_male.",graph_format,sep="")
  open_plot(graph_format=graph_format,file=bild1,width=7,height=5)
  z<-graph.hist(data=dmbm,T0=0,T=T,x1=x1,x2=x2,main="male",bw=bwm,step=stepm,xlab=paste(testvalue," ",testunit))
  close_plot(graph_format=graph_format,file=bild1)
  close_plot(graph_format=graph_format,file=bild1.1)
  modm<-round(z$mod,2)
  bild2<-paste(result_file,"kde_female.",graph_format,sep="")
  bild2.1<-paste(result_file1,"/","kde_female.",graph_format,sep="")
  open_plot(graph_format=graph_format,file=bild2,width=7,height=5)
  z<-graph.hist(data=dwbm,T0=0,T=T,x1=x1,x2=x2,main="female",bw=bww,step=stepw,xlab=paste(testvalue," ",testunit))
  close_plot(graph_format=graph_format,file=bild2)
  close_plot(graph_format=graph_format,file=bild2.1)
  modw<-round(z$mod,2)
  bild3<-paste(result_file,"kde_all.",graph_format,sep="")
  bild3.1<-paste(result_file1,"/","kde_all.",graph_format,sep="")
  open_plot(graph_format=graph_format,file=bild3,width=7,height=5)
  z<-graph.hist(data=dbm,T0=0,T=T,x1=x1,x2=x2,main="all",bw=bwa,step=stepa,xlab=paste(testvalue," ",testunit))
  close_plot(graph_format=graph_format,file=bild3)
  close_plot(graph_format=graph_format,file=bild3.1)
  mod<-round(z$mod,2)
  ######################
  h0<-c("male","female","all")
  h0.age<-rep(paste(a1,"-",a2),3)
  h1<-c(n2,n3,n1)
  h2<-c(modm,modw,mod)
  h3<-c(mean2,mean3,mean1)
  h4<-c(med2,med3,med1)
  h5<-c(sd2,sd3,sd1)
  h6<-c(min2,min3,min1)
  h7<-c(max2,max3,max1)
  d<-data.frame(h0,h0.age,h1,h2,h3,h4,h5,h6,h7)
  colnames(d)<-c("Pop.","Age","N","Mode","Mean","Median","SD","Minimun","Maximum")
  write.table(d, file = flag_file,quote=F,sep=";",col.names=TRUE)
  write.csv(d, file = savename1,quote=F)
  Result1<-paste(result_file1,"/","result1.csv",sep="")
  write.csv(d, file = Result1,quote=F)
  ########################
  })
########################
#########################
errmess<-geterrmessage()                                                                       # get error message 
if (errmess=="" | (!is.na(bw0) & !is.na(bw1) & !is.na(bw2)))  {
  errmess<-""
  } else {
  errmess<-"error: either entries in parameter for analysis are not defined correctely or data in each line are not separated by ";" ."
  }
sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
end.time <- date()
cat("Program start:", start.time,"\n")
cat("program end:  ", end.time,"\n")
cat("================================================================\n\n")
cat(errmess, "\n")
sink()
write.csv(errmess, file=error_file,quote=FALSE, row.names = FALSE)
write.table(errmess, file=flag_file,quote=FALSE, row.names = FALSE)
sink(file=logdat,append=TRUE, split=TRUE,type = c("output","message"))
cat("================================================================\n\n")
cat("R_program1 finished\n\n")
cat("================================================================\n")
sink()
quit(save = "no", status = 0, runLast = FALSE)